#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""A drop-in substitute for the default ``django.core.cache`` object. This
implementation is using a modified mint cache approach based on
http://www.djangosnippets.org/snippets/793/. Mint caching enables you to
mitigate the so-called dogpile effect (also known as the thundering herd
problem).

In short, it appears when generating a cached value takes long. In that
situation the first request that encounters a cache miss, starts regenerating
the value. Before it manages to finish and put the new value in the cache
backend, other requests come in and also start the costly regeneration process
(because they all saw a cache miss). This is a considerable waste of server
resources and increases the amount of users affected by cache misses.

The solution implemented by this module is to notify a single request of the
cache miss and then to keep serving stale values to subsequent requests until
the notified request finishes updating the value.

.. note::

    This implementation stores additional data along every cached value to
    enable the functionality described above. This means that trying to access
    those values using the regular ``django.core.cache`` will return a tuple
    instead of a bare value.

    This implementation does not support specifying fallback values for the
    ``get()`` function since it would not notify client code that a value is
    stale and has to be updated. Consequently, the dogile effect would occur
    anyway.

Configured by three values in ``settings.py``:

* ``CACHE_DEFAULT_TIMEOUT`` - how long a set value should be considered valid
  (in seconds). After this period the value is considered *stale*, a single
  request starts to update it, and subsequent requests get the old value until
  the value gets updated. **Default**: 300 seconds.

* ``CACHE_FILELOCK_PATH`` - path to a non-existant file which will work as an
  interprocess lock to make cache access atomic. **Default**:
  ``/tmp/langacore_django_cache.lock``

* ``CACHE_MINT_DELAY`` - an upper bound on how long a value should take to be
  generated (in seconds). This value is used for *stale* keys and is the real
  time after which the key is completely removed from the cache. **Default**:
  30 seconds.
"""

# More info in the documentation at 
# http://packages.python.org/lck.django/

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import time

from django.core.cache import cache
from django.conf import settings

from lck.concurrency import synchronized

CACHE_MINT_DELAY = getattr(settings, 'CACHE_MINT_DELAY', 30)
CACHE_DEFAULT_TIMEOUT = getattr(settings, 'CACHE_DEFAULT_TIMEOUT', 300)
CACHE_FILELOCK_PATH = getattr(settings, 'CACHE_FILELOCK_PATH',
    '/tmp/langacore_django_cache.lock')


@synchronized(path=CACHE_FILELOCK_PATH)
def get(key, invalidator=None):
    """Get a value from the cache.

    :param key: the key for which to return the value

    :param invalidator: if the value was set with an invalidator, this
                        parameter should contain a current value for it.
                        If the stored value differs from the current, the
                        cached value is considered invalid just like with
                        a regular timeout.
    """

    packed_val = cache.get(key)
    if packed_val is None:
        return None
    val, val_inv, refresh_time, is_stale = packed_val
    if is_stale:
        return val
    if time.time() > refresh_time or (invalidator and val_inv != invalidator):
        # Store the stale value while the cache revalidates for another
        # CACHE_MINT_DELAY seconds.
        set(key, val, invalidator=None, timeout=CACHE_MINT_DELAY,
            _is_stale=True)
        return None
    return val


@synchronized(path=CACHE_FILELOCK_PATH)
def set(key, val, invalidator=None, timeout=CACHE_DEFAULT_TIMEOUT,
    _is_stale=False):
    """Set a value in the cache.

    :param key: the key under which to set the value

    :param val: the value to set

    :param invalidator: additional data that render a cached value invalid
                        if different on ``cache.get()``

    :param timeout: how long should this value be valid, by default
                    CACHE_DEFAULT_TIMEOUT

    :param _is_stale: boolean, used internally to set a stale value in the
                      cache back-end. Don't use on your own.
    """
    refresh_time = timeout + time.time()
    # if not stale, add the mint delay to the actual refresh so we can have
    # the value stored a bit longer in the backend than it would be otherwise
    real_refresh = timeout if _is_stale else timeout + CACHE_MINT_DELAY
    packed_val = (val, invalidator, refresh_time, _is_stale)
    return cache.set(key, packed_val, real_refresh)


@synchronized(path=CACHE_FILELOCK_PATH)
def delete(key):
    """Removes a value from the cache.

    :param key: the key to delete from the cache
    """

    return cache.delete(key)



import os
from django.core.management.base import NoArgsCommand
from optparse import make_option

class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('--plain', action='store_true', dest='plain',
            help='Tells Django to use plain Python, not bpython/IPython.'),
    )
    help = ("Runs a Python interactive interpreter. Tries to use bpython or "
            "IPython, if it's available.")

    requires_model_validation = False

    def handle_noargs(self, **options):
        # XXX: (Temporary) workaround for ticket #1796: force early loading of all
        # models from installed apps.
        from django.db.models.loading import get_models, get_apps
        loaded_models = get_models()

        use_plain = options.get('plain', False)

        from django.conf import settings
        imported_objects = {'settings': settings}

        import_messages = []
        for app_mod in get_apps():
            app_models = get_models(app_mod)
            if not app_models:
                continue
            model_labels = []
            for model in app_models:
                name = model.__name__
                while name in imported_objects:
                    name += '_'
                imported_objects[name] = model
                if model.__name__ == name:
                    model_labels.append(name)
                else:
                    model_labels.append("{} as {}".format(model.__name__, name))
            import_messages.append("Models from '%s': %s"
                "" % (app_mod.__name__.split('.')[-2], ", ".join(model_labels)))
        try:
            if use_plain:
                # Don't bother loading IPython, because the user wants plain
                # Python.
                raise ImportError
            try:
                from tempfile import mkstemp
                _, tmp_name = mkstemp(suffix='.py')
                tmp = open(tmp_name, 'w')

                try:
                    tmp.write("\n".join((('raise Warning, "%s"' if
                        line.startswith("Failed") else 'print "%s"') % line for
                        line in import_messages)))
                finally:
                    tmp.close()

                try:
                    from bpython import cli
                    cli.main(args=['--interactive', tmp_name],
                        locals_=imported_objects)
                finally:
                    os.unlink(tmp_name)
            except ImportError:
                try:
                    from IPython.frontend.terminal.embed import TerminalInteractiveShell
                    shell = TerminalInteractiveShell()
                    shell.mainloop()
                except ImportError:
                    # IPython < 0.11
                    # Explicitly pass an empty list as arguments, because otherwise
                    # IPython would use sys.argv from this script.
                    try:
                        from IPython.Shell import IPShell
                        shell = IPShell(argv=[])
                        shell.mainloop()
                    except ImportError:
                        # IPython not found at all, raise ImportError
                        raise
        except ImportError:
            import code
            # Set up a dictionary to serve as the environment for the shell, so
            # that tab completion works on objects that are imported at runtime.
            # See ticket 5082.
            for msg in import_messages:
                print msg
            try: # Try activating rlcompleter, because it's handy.
                import readline
            except ImportError:
                pass
            else:
                # We don't have to wrap the following import in a 'try', because
                # we already know 'readline' was imported successfully.
                import rlcompleter
                readline.set_completer(
                    rlcompleter.Completer(imported_objects).complete)
                readline.parse_and_bind("tab:complete")

            # We want to honor both $PYTHONSTARTUP and .pythonrc.py, so follow
            # system conventions and get $PYTHONSTARTUP first then import user.
            if not use_plain:
                pythonrc = os.environ.get("PYTHONSTARTUP")
                if pythonrc and os.path.isfile(pythonrc):
                    try:
                        execfile(pythonrc)
                    except NameError:
                        pass
                # This will import .pythonrc.py as a side-effect
                import user
            code.interact(local=imported_objects)



# -*- coding: utf-8 -*-
### NOTE:
### This snippet will be executed in the ``settings.py`` context and is not an
### actual Python module that you would import in a traditional sense. When you
### do import its name, you get the path to this file so you can ``execfile()``
### it.

if 'CURRENT_DIR' not in locals():
    raise ValueError, ("lck.django.profile_support requires "
                       "current_dir_support.")

import os, os.path
import sys
from optparse import make_option
import django.core.management.base
from django.core.management.base import BaseCommand
from django.core.management.base import handle_default_options as \
                                        original_default_options

local_profile = os.environ.get('DJANGO_SETTINGS_PROFILE', 'local')

if SETTINGS_PATH_MODE == 'flat':
    local_settings = '%s-%s.py' % (SETTINGS_PATH_PREFIX, local_profile)
elif SETTINGS_PATH_MODE == 'nested':
    local_settings = '%s%s%s.py' % (SETTINGS_PATH_PREFIX, os.sep,
                                    local_profile)
else:
    raise ValueError, ("Unsupported settings path mode '%s'"
                       "" % SETTINGS_PATH_MODE)

if os.path.exists(local_settings):
    execfile(local_settings)
elif local_profile != 'local':
    raise ValueError, "%s does not exist." % local_settings

if not hasattr(BaseCommand, '_lck_patched'):
    BaseCommand.option_list = (BaseCommand.option_list +
        (make_option('--profile', help='Name of a settings module, e.g. '
            '"debug", "syncdb", etc. If this isn\'t provided, "local" profile '
            'is assumed.'),))
    BaseCommand._lck_patched = True

    def handle_default_options(options):
        if options.profile:
            os.environ['DJANGO_SETTINGS_PROFILE'] = options.profile
        original_default_options(options)

    django.core.management.base.handle_default_options = handle_default_options



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django import template
from django.conf import settings
from lck.django.common.templatetags import DynamicNode, validate_param_number
from lck.git import get_version

register = template.Library()


@register.tag
def git_version(parser, token):
    args = token.split_contents()
    is_valid = len(args) == 1
    validate_param_number(is_valid, args[0], "no arguments")
    return GitVersionNode()


class GitVersionNode(DynamicNode):
    def render(self, context):
        return get_version(settings.CURRENT_DIR)



# -*- coding: utf-8 -*-
### NOTE:
### This snippet will be executed in the ``settings.py`` context and is not an
### actual Python module that you would import in a traditional sense. When you
### do import its name, you get the path to this file so you can ``execfile()``
### it.

import django.core.management


def django_core_management__find_management_module(app_name):
    """
    Determines the path to the management module for the given app_name,
    without actually importing the application or the management module.

    Raises ImportError if the management module cannot be found for any reason.

    Supports namespace packages.
    """
    import os.path

    parts = app_name.split('.')
    parts.append('management')
    parts = [str(p) for p in parts]

    management_module = __import__(app_name + '.management', fromlist=parts)
    path = management_module.__file__

    if path.endswith(('__init__.py', '__init__.pyc')):
        path = os.path.split(path)[0]

    return path


# PATCH: a more generic find_management_module that supports namespace
# packages.
# COMPATIBILITY: Django 1.1.0 - 1.4.0
django.core.management.find_management_module = \
    django_core_management__find_management_module



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.template import Library, Node, TemplateSyntaxError
from django.conf import settings

register = Library()

class SettingsNode( Node ):
    """Taken from http://www.nomadjourney.com/2009/04/how-to-make-the-django-500-server-error-view-useful/"""
    def __init__( self, setting_names ):
        self.setting_names = setting_names

    def render( self, context ):
        for setting_name in self.setting_names:
            try:
                val = settings.__getattr__( setting_name )
            except AttributeError:
                val = None
            context[ setting_name ] = val
        return ''

def do_settings( parser, token ):
    bits = token.split_contents()
    if not len( bits ) > 1:
        raise TemplateSyntaxError( '%r tag requires at least one argument' % bits[ 0 ] )
    return SettingsNode( bits[ 1: ] )

do_settings = register.tag( 'settings', do_settings )




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""A couple of useful middleware classes related to common framework
functionality (including contrib)."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import cProfile
from operator import add
import re
from time import time

from django.conf import settings
from django.db import connection
from django.http import HttpResponse
from django.utils import translation


INSTRUMENTATION_DIR = getattr(settings, 'INSTRUMENTATION_DIR', '.')
INSTRUMENTATION_RULE = getattr(settings, 'INSTRUMENTATION_RULE',
    lambda request: 'instrumentation' in request.session)


class TimingMiddleware(object):
    """Adds timing information as an HTTP header (named ``X-Slo``) and
    optionally at the end of the body if it ends with ``<!--STATS-->``.

    For the measurements to be most realistic, this middleware should be the
    first on the list. See:

    https://docs.djangoproject.com/en/dev/topics/http/middleware/

    NOTE: If you use any compression middleware (e.g. GZip), the request body
    will not be updated. Timing information is still available in the HTTP
    header. If you still need it to show up in the body, move the compression
    middleware above ``TimingMiddleware`` in the list.

    Loosely based on https://code.djangoproject.com/wiki/PageStatsMiddleware
    and http://lurkingideas.net/profiling-django-projects-cachegrind/
    """

    def _instr_path(self, request, timing_start):
        return "{}/{}-{}".format(INSTRUMENTATION_DIR,
            request.META['PATH_INFO'].strip('/').replace('/', '|'),
            timing_start)

    def process_request(self, request):
        request.META['TIMING_MIDDLEWARE_START'] = time()
        request.META['TIMING_MIDDLEWARE_QUERY_OFFSET'] = len(connection.queries)
        if INSTRUMENTATION_RULE(request):
            request.instrumentation = cProfile.Profile()
            request.instrumentation.enable()

    def process_response(self, request, response):
        n = request.META.get('TIMING_MIDDLEWARE_QUERY_OFFSET', 0)
        start = request.META.get('TIMING_MIDDLEWARE_START')
        if start:
            totTime = time() - start
        else:
            totTime = 0
        queries = len(connection.queries) - n
        if queries:
            dbTime = reduce(add, [float(q['time']) for q in
                connection.queries[n:]])
        else:
            dbTime = 0.0
        pyTime = totTime - dbTime
        stats = {
            'totTime': "%.2f" % totTime,
            'pyTime': "%.2f" % pyTime,
            'dbTime': "%.2f" % dbTime,
            'queries': "%d" % queries,
        }
        stat_fmt = "{totTime}"
        if settings.DEBUG:
            stat_fmt += (" Code: {pyTime} DB: {dbTime} Q: "
                "{queries}")
        stat_fmt = stat_fmt.format(**stats)
        if response and response.content:
            content = response.content.rstrip()
            if content.endswith(b"<!--STATS-->"):
                content = content.decode(response._charset)
                content = content[:-12] + ('<!-- Total: ' + stat_fmt + ' -->\n')
                response.content = content.encode(response._charset)
        response['X-Slo'] = stat_fmt
        if hasattr(request, 'instrumentation'):
            path = self._instr_path(request, start)
            request.instrumentation.disable()
            request.instrumentation.dump_stats('{}-{}-{}.prof'.format(path,
                request.user.username or 'anon', stats['totTime']))
        return response


class ForceLanguageCodeMiddleware(object):
    """Add this middleware to force the whole website to always use the
    language specified in ``settings.LANGUAGE_CODE`` instead of sniffing it
    from the user agent."""

    def process_request(self, request):
        request.LANG = settings.LANGUAGE_CODE
        translation.activate(request.LANG)
        request.LANGUAGE_CODE = request.LANG


class AdminForceLanguageCodeMiddleware(object):
    """Add this middleware to force the admin to always use the language
    specified in ``settings.LANGUAGE_CODE`` instead of sniffing it from
    the user agent."""

    def process_request(self, request):
        if request.path.startswith('/admin'):
            request.LANG = settings.LANGUAGE_CODE
            translation.activate(request.LANG)
            request.LANGUAGE_CODE = request.LANG


class BasicAuthMiddleware(object):
    def unauthorized(self):
        response = HttpResponse("""<!DOCTYPE html>
        <html>
            <title>Who's there?</title>
            <body>
                <h1>Authorization Required</h1>
            </body>
        </html>""", mimetype="text/html")
        response['WWW-Authenticate'] = 'Basic realm="Development"'
        response.status_code = 401
        return response

    def process_request(self, request):
        for pattern in settings.BASICAUTH_PUBLIC:
            if re.match(pattern, request.META['PATH_INFO']):
                return
        if 'HTTP_AUTHORIZATION' not in request.META:
            return self.unauthorized()
        authentication = request.META['HTTP_AUTHORIZATION']
        try:
            authmeth, auth = authentication.split(' ', 1)
        except Exception:
            # Russian script kiddies and their invalid headers
            return self.unauthorized()
        if authmeth.lower() != 'basic':
            return self.unauthorized()
        auth = auth.strip().decode('base64')
        try:
            username, password = auth.split(':', 1)
        except UnicodeDecodeError:
            # Russian script kiddies and their strange encodings
            return self.unauthorized()
        if (username, password) == (settings.BASICAUTH_USERNAME,
                                    settings.BASICAUTH_PASSWORD):
            return
        return self.unauthorized()


class SessionAwareLanguageMiddleware(object):
    """If the request is provided with a ?lang GET argument, change
    the language accordingly."""

    def process_request(self, request):
        if 'lang' in request.GET:
            request.session['_language_code'] = request.GET['lang']
        if '_language_code' in request.session:
            request.LANG = request.session['_language_code']
            translation.activate(request.LANG)
            request.LANGUAGE_CODE = request.LANG



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""lck.django.badges.models
   ------------------------

   Models holding badges."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import importlib
from os.path import splitext

from django.db import models as db
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.generic import GenericForeignKey
from django.utils.translation import ugettext_lazy as _
from lck.django.common.models import Named, TimeTrackable, EditorTrackable,\
    ImageModel


class BadgeMetadata(Named.NonUnique, TimeTrackable):
    key = db.CharField(verbose_name=_("key"), max_length=75, primary_key=True)
    description = db.TextField(verbose_name=_("description"),
        null=True, blank=True, default=None)
    callback = db.CharField(verbose_name=_("Python callback"), max_length=100,
        help_text=_("path to a Python function that will award/discard badges"),
        blank=True, default="")

    class Meta:
        abstract = True

    def __unicode__(self):
        return "{} ({})".format(self.name, self.key)


class BadgeGroup(BadgeMetadata):
    """Owners can be awarded only a single badge of a specified group at
    a time."""
    multiple_allowed = db.BooleanField(verbose_name=_("multiple badges of "
        "this group allowed?"), default=False)

    class Meta:
        verbose_name = _("badge group")
        verbose_name_plural = _("badge groups")


class BadgeIcon(ImageModel):
    """Image representation of the badge."""
    image = ImageModel.image_field(upload_to='badge_icons')

    class Meta:
        verbose_name = _("badge icon")
        verbose_name_plural = _("badge icons")


class BadgeType(BadgeMetadata):
    """Describes a specific badge type."""
    group = db.ForeignKey(BadgeGroup, verbose_name=_("badge group"))
    icon = db.ForeignKey(BadgeIcon, verbose_name=_("badge icon"),
        blank=True, null=True, default=None)

    class Meta:
        verbose_name = _("badge type")
        verbose_name_plural = _("badge types")

    def get_description(self):
        return self.description or self.group.description or ""


class Badge(TimeTrackable, EditorTrackable):
    """A single badge given to an `owner` for a certain action on
    a `subject`."""
    type = db.ForeignKey(BadgeType, verbose_name=_("badge type"))
    owner_ct = db.ForeignKey(ContentType, verbose_name=_("owner (content type)"),
        related_name="%(app_label)s_%(class)s_owned_received")
    owner_oid = db.IntegerField(verbose_name=_("owner (instance id)"),
        db_index=True)
    owner = GenericForeignKey('owner_ct', 'owner_oid')
    subject_ct = db.ForeignKey(ContentType, verbose_name=_("subject (content type)"),
        related_name="%(app_label)s_%(class)s_badges_given",
        null=True, blank=True, default=None)
    subject_oid = db.IntegerField(verbose_name=_("subject (instance id)"),
        db_index=True, null=True, blank=True, default=None)
    subject = GenericForeignKey('subject_ct', 'subject_oid')
    comment = db.TextField(verbose_name=_("editor comment"),
        null=True, blank=True, default=None)

    class Meta:
        verbose_name = _("badge")
        verbose_name_plural = _("badges")

    def __unicode__(self):
        return "{} (o: {}, s: {})".format(self.type, self.owner, self.subject)

    @classmethod
    def find_by_group(cls, group, owner, subject=None):
        return cls._find_badge('type__group__key', group, owner, subject)

    @classmethod
    def find_by_type(cls, type, owner, subject=None):
        return cls._find_badge('type__key', type, owner, subject)

    @classmethod
    def find_by_owner(cls, owner, subject=None):
        return cls._find_badge(None, None, owner, subject)

    @classmethod
    def _find_badge(cls, key, value, owner, subject=None):
        filter = {
            'owner_ct': ContentType.objects.get_for_model(owner.__class__),
            'owner_oid': owner.pk,
        }
        if value is not None:
            filter[key] = value
        if subject:
            filter['subject_ct'] = ContentType.objects.get_for_model(
                subject.__class__)
            filter['subject_oid'] = subject.pk
        badge = Badge.objects.filter(**filter)
        if badge.exists():
            return badge
        return None

    @classmethod
    def _create(cls, type, owner, subject):
        # convoluted because of Django bugs #7551 and #12316
        badge = Badge(type=type)
        badge.owner = owner
        if subject:
            badge.subject = subject
        return badge

    @classmethod
    def award(cls, type, owner, subject=None):
        """Award a badge of a specified `type` to an `owner` for actions
        on a specific `subject`.

        If there is an existing badge of the same group and the same type,
        no other badge is awarded unless `multiple_allowed` is set on the
        badge group.

        If there is an existing badge of the same group but of a different
        type, it is "upgraded" to the current type unless `multiple_allowed`
        is set on the badge group. In the latter case a duplicate badge is
        awarded.
        """
        badge_type = BadgeType.objects.get(pk=type)
        old_badges = Badge.find_by_group(badge_type.group.key, owner=owner,
            subject=subject)
        create_badge = False
        if badge_type.group.multiple_allowed or not old_badges:
            create_badge = True
        else:
            # delete badge versions which are to be upgraded
            old_badges.exclude(type=badge_type).delete()
            if old_badges.exists():
                # leave a single badge
                for outdated in old_badges[1:]:
                    outdated.delete()
                return old_badges[0]
            else:
                create_badge = True
        if create_badge:
            badge = Badge._create(type=badge_type, owner=owner, subject=subject)
            badge.save()
            return badge
        return None


def update_type(type, *args, **kwargs):
    """Send a badge type callback task through Celery."""
    badge_type = BadgeType.objects.get(pk=type)
    _do_update(badge_type.callback or badge_type.group.callback, *args, **kwargs)


def update_group(group, *args, **kwargs):
    """Send a badge group callback task through Celery."""
    _do_update(BadgeGroup.objects.get(pk=group).callback, *args, **kwargs)


def _do_update(callback, *args, **kwargs):
    module, function = splitext(callback)
    mod = importlib.import_module(module)
    try:
        eval('mod{}.delay(*args, **kwargs)'.format(function))
    except (NameError, AttributeError):
        raise ValueError("Cannot find badge callback `{}`".format(callback))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


from django.db import models as db
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _
from lck.django.common.models import Titled, Localized, TimeTrackable


class FlatPage(Titled.NonUnique, Localized, TimeTrackable):
    url = db.CharField(_('URL'), max_length=100, db_index=True)
    content = db.TextField(_('content'), blank=True)
    enable_comments = db.BooleanField(_('enable comments'))
    template_name = db.CharField(_('template name'), max_length=70, blank=True,
        help_text=_("Example: 'flatpages/contact_page.html'. If this isn't "
            "provided, the system will use 'flatpages/default.html'."))
    registration_required = db.BooleanField(_('registration required'),
        help_text=_("If this is checked, only logged-in users will be able to "
            "view the page."))
    sites = db.ManyToManyField(Site)

    class Meta:
        verbose_name = _('flat page')
        verbose_name_plural = _('flat pages')
        ordering = ('url', 'language')
        unique_together = ('url', 'language')

    def __unicode__(self):
        return u"{} ({}) -- {}".format(self.url, self.get_language_display(),
            self.title)

    def get_absolute_url(self):
        return self.url



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""The thumbnail filter."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os

from django import template
from PIL import Image


register = template.Library()


def _is_proper_image(image):
    if image is not None:
        for attr in 'width', 'height', 'file', 'url':
            if not hasattr(image, attr):
                return False
        return True
    return False


@register.filter
def thumbnail(image, size):
    """For a given `image` constructs a thumbnail using the rule specified in
    the `size` param.

    Supported types of `image`:

    * any object with the `width`, `height`, `file` and `url` attributes (such
      as a Django ImageField)

    * any object with an `image` attribute pointing to an object of the other
      kind

    If an incompatible object is passed, None is returned.

    Possible values for `size`:

    * h{height} - scale to match the height given. Example: "h100" scales to
                  100px in height. Width is scaled proportionally, may exceed
                  100px.

    * w{width} - scale to match the width given. Example: "w312" scales to
                 312px in width. Height is scaled proportionally, may exceed
                 312px.

    * m{max_size} - scale to match the greater dimension to the value given.
                    Example: "m50" would scale an 80x50 image to 50x31, and
                    an 40x120 image to 17x50.

    * s{max_size} - cuts a square from the image and scales it to the value
                    given. Example: "s48" would scale a 50x80 image to 48x48
                    based on the top 50x50 square.

    The thumbnails are kept next to the original images with a `_{size}` suffix
    added to the name. If a thumbnail exists and is newer than the original
    image, it is reused on subsequent calls."""

    if not _is_proper_image(image):
        if hasattr(image, 'image') and _is_proper_image(image.image):
            image = image.image
        else:
            return None

    shift = None
    if size.startswith("h"):
        height = int(size[1:])
        width = height * image.width / image.height
    elif size.startswith("w"):
        width = int(size[1:])
        height = width * image.height / image.width
    elif size.startswith("m"):
        max = int(size[1:])
        if image.width > image.height:
            width = max
            height = width * image.height / image.width
        else:
            height = max
            width = height * image.width / image.height
    elif size.startswith("s"):
        width = height = int(size[1:])
        if image.width > image.height:
            shift = int((image.width - image.height) / 2)
        else:
            shift = 0
    else:
        width, height = size.split("x")
    width = int(round(width))
    height = int(round(height))

    file = image.file.name
    try:
        basename, format = file.rsplit('.', 1)
    except ValueError: # if there is no extension
        basename = file
        format = 'jpg'
    thumb_filename = basename + '_' + size + '.' + format

    try:
        baseurl, format = image.url.rsplit('.', 1)
    except ValueError: # if there is no extension
        baseurl = image.url
        format = 'jpg'
    thumb_url = baseurl + '_' + size + '.' + format

    if os.path.exists(thumb_filename):
        if os.path.getmtime(thumb_filename) < os.path.getmtime(file):
            create_new = True
            os.unlink(thumb_filename)
        else:
            create_new = False
    else:
        create_new = True

    if create_new:
        pil_image = Image.open(file)
        pil_format = pil_image.format or 'JPEG'
        if pil_format not in Image.SAVE:
            pil_format = 'JPEG'
        if shift is not None:
            shorter = min(image.width, image.height)
            pil_image = pil_image.crop((shift, 0, shorter+shift, shorter))
        pil_image.thumbnail((width, height), Image.ANTIALIAS)
        if pil_format == 'JPEG' and pil_image.mode != 'RGB':
            pil_image = pil_image.convert('RGB')
        pil_image.save(thumb_filename, pil_format)

    return thumb_url



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""lck.django.common.templatetags
   ------------------------------

   Commonly useful template tags and filters."""
#FIXME: Elaborate what's included.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from functools import partial

from django.template import Node, Context, RequestContext, Template,\
    TemplateSyntaxError

from lck.cache import memoize


def validate_param_number(condition, tag_name, fail_text):
    if not condition:
        raise TemplateSyntaxError("%r tag requires %s argument%s" % (tag_name,
            fail_text, "s" if fail_text != "one" else ""))


def _render(text, context):
    if not context:
        raise ValueError("Provided context is empty.")
    elif 'request' in context:
        return Template(text).render(RequestContext(context['request'],
            context))
    else:
        return Template(text).render(Context(context))


class DynamicNode(Node):
    """A Node that provides a rendering method to support dynamic arguments."""

    render = staticmethod(_render)

    @memoize(skip_first=True)
    def dynamic(self, text, load=[], prefix='', suffix=''):
        """Prepares tag attribute `text` to be rendered using the templating
        engine. Returns a callable with a single `context` attribute.

        If `load` is given, it's a list of tag/filter libraries required
        for rendering of the `text`. `prefix` and `suffix` hold boilerplate
        that needs to be added around the provided text.

        If `text` is empty, the returned callable always returns an empty
        string. In this case the templating engine is not used and the
        `prefix`, `suffix` and `load` attributes are ignored.
        """
        if text:
            load = ['{{%load {}%}}'.format(l) for l in load]
            if prefix:
                load.append(prefix)
            load.append(text)
            if suffix:
                load.append(suffix)
            text = "\n".join(load)
            return partial(_render, text)
        else:
            return lambda context: ""

    def static(self, text, prefix='', suffix=''):
        """Prepares tag attribute `text` to be rendered statically. Unless
        `text` is empty, returns a string surrounded by `prefix` and `suffix`.
        Otherwise returns an empty string."""
        if text:
            return prefix + text + suffix
        else:
            return ''



# -*- coding: utf-8 -*-
from south.v2 import SchemaMigration

class Migration(SchemaMigration):

    def forwards(self, orm):
        pass

    def backwards(self, orm):
        pass

    models = {}

    complete_apps = ['common']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from httplib import HTTPException
from optparse import make_option
from urllib2 import Request, build_opener, URLError

from django.conf import settings
from django.core.management.base import NoArgsCommand
from lck.django.activitylog.models import Backlink, BacklinkStatus

from __builtin__ import print as builtin_print

def print(*args, **kwargs):
    builtin_print(*[s.encode('utf8') for s in args], **kwargs)


BACKLINK_VERIFICATION_USER_AGENT = getattr(settings,
    'BACKLINK_VERIFICATION_USER_AGENT', 'Mozilla/5.0 (X11; Linux x86_64; '
    'rv:7.0.1) Gecko/20100101 Firefox/7.0.1 lck.django')


class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('--all', action='store_true', dest='all',
            help='Verifies also previously verified backlinks.'),
    )
    help = ("Verifies backlinks by following the URL and checking whether "
            "a link back actually exists.")

    def handle_noargs(self, **options):
        if isinstance(BACKLINK_VERIFICATION_USER_AGENT, unicode):
            bvua = BACKLINK_VERIFICATION_USER_AGENT.encode('utf8')
        else:
            bvua = BACKLINK_VERIFICATION_USER_AGENT
        verify_all = options.get('all', False)
        backlinks = Backlink.objects.select_related()
        if verify_all:
            # Merged backlinks won't probably correctly verify but they have
            # been merged from verified backlinks anyway.
            backlinks = backlinks.exclude(status=BacklinkStatus.merged.id)
        else:
            backlinks = backlinks.filter(status__in=BacklinkStatus.is_verifiable())
        url_opener = build_opener()
        for backlink in backlinks:
            try:
                url = Request(backlink.referrer.encode('utf8'))
                url.add_header(b'User-Agent', bvua)
                data = url_opener.open(url, timeout=20).read()
            except (URLError, IOError, HTTPException):
                verified = False
            else:
                verified = backlink.site.domain.encode('utf8') in data
            if verified:
                # don't have to care about merged since we exclude them anyway
                backlink.status = BacklinkStatus.verified.id
            else:
                if backlink.status in BacklinkStatus.is_verified():
                    backlink.status = BacklinkStatus.verification_failed1.id
                elif backlink.status in BacklinkStatus.can_increment_failure_status():
                    backlink.status += 1
                else:
                    backlink.status = BacklinkStatus.failed.id
            if 'status' in backlink.dirty_fields:
                print(backlink.referrer, 'for URL', backlink.url,
                    '({} visits)'.format(backlink.visits),
                    'is' if backlink.status == BacklinkStatus.verified.id \
                         else 'IS NOT', 'verified.')
                backlink.save()



from django import template
from django.template.defaultfilters import stringfilter

register = template.Library()

def varname(value):
    return value.lower().replace("-","_")

register.filter(varname)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""lck.django.profile.models
   -------------------------

   Contains building blocks for creating user profiles."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from datetime import date
from functools import partial
from hashlib import md5
import re

from django.contrib.auth.models import User
from django.db import models as db
from django.utils.translation import ugettext_lazy as _
from dj.choices import Country, Gender

from lck.django.common.templatetags.thumbnail import thumbnail

TZ_CHOICES = [(float(x[0]), x[1]) for x in (
    (-12, '-12'), (-11, '-11'), (-10, '-10'), (-9.5, '-09.5'), (-9, '-09'),
    (-8.5, '-08.5'), (-8, '-08 PST'), (-7, '-07 MST'), (-6, '-06 CST'),
    (-5, '-05 EST'), (-4, '-04 AST'), (-3.5, '-03.5'), (-3, '-03 ADT'),
    (-2, '-02'), (-1, '-01'), (0, '00 GMT'), (1, '+01 CET'), (2, '+02'),
    (3, '+03'), (3.5, '+03.5'), (4, '+04'), (4.5, '+04.5'), (5, '+05'),
    (5.5, '+05.5'), (6, '+06'), (6.5, '+06.5'), (7, '+07'), (8, '+08'),
    (9, '+09'), (9.5, '+09.5'), (10, '+10'), (10.5, '+10.5'), (11, '+11'),
    (11.5, '+11.5'), (12, '+12'), (13, '+13'), (14, '+14'),
)]

AVATAR_ATTR_REGEX = re.compile(r"^avatar_([whms])(\d+)$")


class BasicInfo(db.Model):
    """Describes a basic user profile that links through a one-to-one field
    to ``django.contrib.auth.models.User``. Provides fields to store nicks,
    birth dates, genders, countries of origin, cities and time zones.

    This model is also a transparent proxy to the User object enabling most
    of the functionality that would require explicitly getting the user model.
    This enables for enhanced duck typing in scenarios where user objects
    are expected."""
    user = db.OneToOneField(User)
    nick = db.CharField(verbose_name=_("visible nick"), blank=True, default='',
        max_length=30, help_text=_("Fill this field if you wish to change "
            "your visible nick. You can use Unicode characters and spaces. "
            "Keep in mind that your original username (the one you use to "
            "log into the site) will remain unchanged."))
    birth_date = db.DateField(verbose_name=_("birth date"), blank=True,
        null=True)
    gender = db.PositiveIntegerField(verbose_name=_("gender"),
        choices=Gender(), default=Gender.male.id)
    country = db.PositiveIntegerField(verbose_name=_("country"),
        choices=Country(), default=Country.pl.id)
    city = db.CharField(verbose_name=_("city"), max_length=30, blank=True)
    time_zone = db.FloatField(_('Time zone'), choices=TZ_CHOICES,
        default=1.0)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if not self.nick.strip():
            self.nick = self.user.username
        super(BasicInfo, self).save(*args, **kwargs)

    def __getattr__(self, name):
        try:
            return super(BasicInfo, self).__getattr__(name)
        except AttributeError:
            return getattr(User.objects.get(id=self.__dict__['user_id']), name)

    def age(self):
        """age() -> numeric_age"""
        if not self.birth_date:
            return '?'
        else:
            return int((date.today() - self.birth_date).days / 365.25)

    def is_male(self):
        """True if the gender is male."""
        return self.gender == Gender.male.id

    def is_female(self):
        """True if the gender is female."""
        return self.gender == Gender.female.id

    def has_gender(self):
        """True if the gender is set."""
        return self.is_male() or self.is_female()

    def get_country_display_english(self):
        """Displays the country of origin in its English form. This is useful
        for distionary lookups."""
        return Country.raw_from_id(self.country)

    def get_profile(self):
        return self

    @property
    def profile(self):
        return self

    def is_authenticated(self):
        return True


class ActivationSupport(db.Model):
    """Adds an ``activation_token`` field."""
    activation_token = db.CharField(verbose_name=_("activation token"),
        max_length=40, editable=False, blank=True, default='')

    class Meta:
        abstract = True


class AvatarSupport(db.Model):
    """Enables getting the user's gravatar by using the ``model.avatar_{size}``
    attribute notation. Works in both code and templates.

    Possible values for `size`:

    * h{height} - scale to match the height given. Example: "h100" scales to
                  100px in height. Width is scaled proportionally, may exceed
                  100px.

    * w{width} - scale to match the width given. Example: "w312" scales to
                 312px in width. Height is scaled proportionally, may exceed
                 312px.

    * m{max_size} - scale to match the greater dimension to the value given.
                    Example: "m50" would scale an 80x50 image to 50x31, and
                    an 40x120 image to 17x50.

    * s{max_size} - cuts a square from the image and scales it to the value
                    given. Example: "s48" would scale a 50x80 image to 48x48
                    based on the top 50x50 square.

    When inheriting, create a new `avatar` field like this::

        avatar = AvatarSupport.avatar_field(upload_to='upload_directory')

    To have gravatar fallback, specify `GravatarSupport` **after**
    `AvatarSupport` in your model inheritance list.
    """
    avatar_height = db.PositiveIntegerField(verbose_name=_("height"),
        default=None, blank=True, null=True, editable=False)
    avatar_width = db.PositiveIntegerField(verbose_name=_("width"),
        default=None, blank=True, null=True, editable=False)
    avatar_field = partial(db.ImageField, verbose_name=_("custom avatar"),
        height_field='avatar_height', width_field='avatar_width',
        null=True, blank=True, max_length=255)

    def __getattr__(self, name):
        m = AVATAR_ATTR_REGEX.match(name)
        if m and self.avatar:
            mode_size = "{}{}".format(m.group(1), m.group(2))
            return thumbnail(self.avatar, mode_size)
        return super(AvatarSupport, self).__getattr__(name)

    class Meta:
        abstract = True


class GravatarSupport(db.Model):
    """Enables getting the user's gravatar by using the ``model.avatar_{size}``
    attribute notation. Works in both code and templates.

    Possible values for `size` are the same as in ``AvatarSupport`` but the
    modifier is always treated as "s".
    """
    def get_gravatar(self, size):
        email = self.email if hasattr(self, 'email') else self.user.email
        return "http://www.gravatar.com/avatar/%s.jpg?d=identicon&s=%d" % (
            md5(email).hexdigest(), size)

    def __getattr__(self, name):
        m = AVATAR_ATTR_REGEX.match(name)
        if m:
            size = int(m.group(2))
            return self.get_gravatar(size)
        return super(GravatarSupport, self).__getattr__(name)

    class Meta:
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Various unrelated routines and helper classes."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from functools import wraps
import re
from zlib import adler32, crc32

from django.conf import settings
from django.db import transaction
from django.http import HttpResponse, HttpResponseRedirect
from django.template import loader, RequestContext
from django.utils import simplejson
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from dj.chain import chain as lazy_chain # compatibility with lck.django < 0.8
from dj.choices import Language
from lck.cache import memoize


DOTS_REGEX = re.compile(r'\.\s+')
INTERNAL_IPS = getattr(settings, 'INTERNAL_IPS', set(('127.0.0.1', '::1',
    'localhost')))


def render(request, template_name, context, debug=False, mimetype=None,
    compute_etag=True):
    """render(request, template_name, context, [debug, mimetype]) -> HttpResponse

    Renders the `context` within for a `request` using a template named
    `template_name` returning the required `mimetype`."""

    if hasattr(settings, 'AUTH_PROFILE_MODULE'):
        if 'user_profile' in context:
            raise KeyError, ("lck.django.common.render() doesn't "
                             "accept contexts with 'user_profile'")
        if 'other_user_profile' in context:
            raise KeyError, ("lck.django.common.render() doesn't "
                             "accept contexts with 'other_user_profile'")

        context['user_profile'] = request.user.get_profile() \
                                  if request.user.is_authenticated() else None
        context['other_user_profile'] = context['other_user'].get_profile() \
                                        if 'other_user' in context else None

    http_response_kwargs = {'mimetype': mimetype}
    response = loader.render_to_string(template_name, RequestContext(request,
        context)).encode('utf8')
    if compute_etag:
        etag = b"-".join((hex(crc32(response)), hex(adler32(response))))
        if etag == request.environ.get('HTTP_IF_NONE_MATCH', 'X'):
            response = ""
            http_response_kwargs['status'] = 304
    http_response = HttpResponse(response, **http_response_kwargs)
    if compute_etag:
        http_response['ETag'] = etag
    return http_response


def render_decorator(func):
    """render_decorator(func) -> HttpResponse

    Wraps a view function returning `context` (most conveniently: locals()).
    Renders the `context` for a context['request'] using a template named
    context['template'] returning the required context['mimetype']."""

    @wraps(func)
    def wrapper(*args, **kwargs):
        context = func(*args, **kwargs)
        return render(context['request'],
                      context['template'],
                      RequestContext(context['request'], context),
                      mimetype=context.get('mimetype', None))
    return wrapper


def render_json(obj):
    """render_json(obj) -> HttpResponse

    Dumps the object in the `obj` param to a JSON string and returns it
    via HTTP using the application/json mimetype."""

    return HttpResponse(simplejson.dumps(obj),
                        mimetype="application/json")


def redirect(request, link='/', override_request=False):
    """redirect(request, [link, override_request]) -> HttpResponseRedirect

    A smarter redirect which takes the redirection target from a given param.
    The lookup for the redirection target is as follows:

    * 'redirect_to' in GET or POST (in that order)

    * 'next' in GET or POST (in that order)

    * the `link` fallback argument ('/' if not given)

    If `override_request` is ``True``, `link` is always followed."""

    if not override_request:
        link_from_param = None
        for varname in 'redirect_to', 'next':
            for container in request.GET, request.POST:
                if varname in container:
                    link_from_param = container[varname]
                    break
            if link_from_param:
                link = link_from_param
                break
    return HttpResponseRedirect(link)


def typical_handler(request, form_class, template, initial={},
                    initial_kwargs={}, context={}, redirect_on_success=None):
    """A handler for a typical form workflow:

    1. Initialize a form object.

       a. if there are POSTed data and/or files, use them
       b. otherwise use `initial` as initial data and `initial_kwargs` as \
          keyword arguments for the form object constructor.

    2. If the form validates, save it and
       either: override the template name with a version using the \
          "_complete.html" suffix.
       or: redirect using the `redirect_on_success` arguments
    3. If there are errors on the form, prepare an additional `error_summary`
       string on the form object (for consumption by the template).
    4. Render the result.
    """
    if request.method == 'POST':
        _kwargs = dict(initial_kwargs)
        _kwargs['data'] = request.POST
        _kwargs['files'] = request.FILES
    else:
        _kwargs = dict(initial_kwargs)
        _kwargs['initial'] = initial
    form = form_class(**_kwargs)
    if form.is_valid():
        template = template + '_complete.html'
        if hasattr(form, 'save'):
            form.request = request
            form.save()
        if redirect_on_success:
            return redirect(request, **redirect_on_success)
    else:
        template += '.html'
        if form.errors:
            error = unicode(" ".join(form.errors.get("__all__",
                                    [_("Try again")])))
            error = DOTS_REGEX.sub('.<br>', error,
                                   len(DOTS_REGEX.findall(error))-1)
            form.error_summary = mark_safe(error)
            del error
    ctx = dict(context)
    ctx.update(locals())
    return render(request, template, ctx)


def cut(text, length=40, trailing=" (...)"):
    """Cuts text to a predefined length and appends a trailing ellipsis for
    longer sources."""
    if not text:
        return text
    if len(text) <= length:
        trailing = ""
    return text[:length] + trailing


def remote_addr(request):
    """If the remote address in request is a localhost, check for
    X_FORWADED_FOR. Which addresses are considered local is defined by the
    ``INTERNAL_IPS`` list in `settings.py`, by default these are 127.0.0.1,
    ::1 and "localhost"."""
    result = request.META['REMOTE_ADDR']
    if result in INTERNAL_IPS:
        try:
            result = request.META['HTTP_X_FORWARDED_FOR'].split(',')[0].strip()
        except (KeyError, IndexError):
            pass
    return result


def nested_commit_on_success(func):
    """Like commit_on_success, but doesn't commit existing transactions.

    This decorator is used to run a function within the scope of a
    database transaction, committing the transaction on success and
    rolling it back if an exception occurs.

    Unlike the standard transaction.commit_on_success decorator, this
    version first checks whether a transaction is already active.  If so
    then it doesn't perform any commits or rollbacks, leaving that up to
    whoever is managing the active transaction.

    Taken from: http://djangosnippets.org/snippets/1343/
    """
    _commit_on_success = transaction.commit_on_success(func)
    def _nested_commit_on_success(*args, **kwds):
        if transaction.is_managed():
            return func(*args,**kwds)
        else:
            return _commit_on_success(*args,**kwds)
    return transaction.wraps(func)(_nested_commit_on_success)


def get_langs(request, custom=None):
    """ get_langs(request) -> list of language IDs

    First there's the exact language as found in the request headers, followed
    by a generic form and by the ``LANGUAGE_CODE`` value from settings."""
    langs = []
    accept = request.META.get('HTTP_ACCEPT_LANGUAGE', '*')
    if accept != '*':
        langs.append({'language': Language.id_from_name(request.LANGUAGE_CODE)})
        langs.append({'language':
            Language.id_from_name(request.LANGUAGE_CODE.split("-")[0])})
    if custom:
        langs.append(custom)
    langs.append({'language': Language.id_from_name(settings.LANGUAGE_CODE)})
    langs.append({'language':
        Language.id_from_name(settings.LANGUAGE_CODE.split("-")[0])})
    return langs


@memoize
def model_is_user(model):
    from django.contrib.auth.models import User
    return model in (User, 'auth.User')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2010 Łukasz Langa

"""Allplay filters."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from functools import partial
import re

from django.conf import settings
from django.template import Library
from django.utils.safestring import mark_safe

from lck.cache import memoize
from lck.django.common.templatetags import DynamicNode, validate_param_number


register = Library()
COLOR_MAP = {
    'red': '31',
    'green': '32',
    'blue': '34',
}


@register.tag
def color(parser, token):
    args = token.split_contents()
    is_simple = len(args) == 2
    is_complex = len(args) == 4 and  args[2] == 'in'
    is_valid = is_simple or is_complex
    validate_param_number(is_valid, args[0], "one or three arguments")
    return ColorNode(args[1], args[3] if is_complex else None)


class ColorNode(DynamicNode):
    def __init__(self, color, from_map=None):
        if not from_map:
            self.color_name = lambda ctx: color
            self.color_map = lambda ctx: COLOR_MAP
        else:
            self.color_name = self.dynamic(color)
            self.color_map = lambda ctx: ctx.get(from_map, {})

    def render(self, context):
        if 'no_color' in context and context['no_color']:
            return ""
        else:
            color = self.color_map(context).get(self.color_name(context),
                '00')
            return "\033[{}m".format(color)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.http import Http404
from django.conf import settings

from lck.django.flatpages.views import flatpage

class FlatpageFallbackMiddleware(object):
    def process_response(self, request, response):
        if response.status_code != 404:
            return response # No need to check for a flatpage for non-404 responses.
        try:
            return flatpage(request, request.path_info)
        # Return the original response if any errors happened. Because this
        # is a middleware, we can't assume the errors will be caught elsewhere.
        except Http404:
            return response
        except:
            if settings.DEBUG:
                raise
            return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""lck.dummy.defaults
   ------------------

   A minimal sample Django app for documentation generation purposes."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.db import models as db
from django.dispatch import receiver
from lck.django.activitylog.models import MonitoredActivity
from lck.django.profile.models import BasicInfo

class Profile(BasicInfo, MonitoredActivity):
    class Meta:
        verbose_name = 'profile'
        verbose_name = 'profiles'


def create_a_user_profile_ignoring_dberrors(instance):
    try:
        profile, new = Profile.objects.get_or_create(user=instance)
        profile.save() # to trigger nick update etc.
    except DatabaseError:
        pass # no such table yet, first syncdb


@receiver(db.signals.post_save, sender=User)
def user_post_save(sender, instance, created, **kwargs):
    if created:
        create_a_user_profile_ignoring_dberrors(instance)


# ensure at start no user comes without a profile
if False: # this is too heavy for tens of thousands of users
    try:
        for u in User.objects.order_by('id'):
            create_a_user_profile_ignoring_dberrors(u)
    except DatabaseError:
        pass # no such table yet, first syncdb
    except Exception as e:
        import traceback
        traceback.print_exc()
        raise SystemExit


# workaround for a unit test bug in Django 1.4.x

from django.contrib.auth.tests import models as auth_test_models
del auth_test_models.ProfileTestCase.test_site_profile_not_available
